int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    while (size) {
        ssize_t r = fread(buf, 1, sizeof(buf) - 1, fp);
        if (r > 0)
            buf[r] = 0;
        if (r <= 0)
            break;
        buf[r] = 0;
        if (strcmp(buf, "P\x06") == 0) {
            log_debug("PS layout: %s %s", buf, lineno);
            ps_reader->parse(buf);
        } else if (strcmp(buf, "FIGURE") == 0) {
            uint8_t *buf_words = malloc(size/32 + 4);
            if (!buf_words) {
                log_error("Could not allocate memory to make headers");
                goto err;
            }
            /* fill the buffer with the words */
            memset(buf_words, 0, size/32 + 4);
            buf_words[0] = 0;
            buf_words[size/32] = 0x40;
            for (size_t i = 0; i < size/32; i++) {
                uint32_t word = ftello(fp);
                memcpy(buf_words+2*i, &word, sizeof(word));
            }
            buf_words[2*size/32] = 0;
            buf_words[size/32+2] = 0;
            parse_figures(&buf_words[size/32+4], size/32+8);
            /* skip to the end of the file */
            fseek(fp, ftello(fp) - 4, SEEK_CUR);
            free(buf_words);
        } else if (strcmp(buf, "EOF") == 0) {
            /* EOF */
            log_debug("PS layout: %s %s", buf, lineno);
            log_debug("EOF");
            parse_figures(NULL, 0);
        } else {
            /* unknown magic data, skip it */
            log_error("unknown PS file: %s %s", buf, lineno);
            goto err;
        }
        size -= r;
    }
    return 0;
err:
    return -1;
}