int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    char *p = buf;
    char *q = out;

    while (size > 0) {
        uint8_t code;
        size_t len = size > 128 ? 128 : size;
        if (len >= 64) {
            return NULL;
        }

        code = *in_data++;
        if (code >= 128) {
            /* check if we have more than 64 chars left in the buffer */
            if (q + len > out) {
                return NULL;
            }

            if (q > out) {
                memcpy(q, in_data - 64, 64);
            }

            in_data += 64;
            size -= 64;
            q += 64;
        }
        if (code > 128) {
            /* check if we have more than 128 chars left in the buffer */
            if (p + len > out) {
                return NULL;
            }

            if (p > out) {
                memcpy(p, in_data - 128, 128);
            }

            in_data += 128;
            size -= 128;
            p += 128;
        }

        if (len < 64) {
            return NULL;
        }

        switch (code) {
            case 128: /* CR */
            case 129: /* LF */
                break;
            case 160: /* DLE */
            case 166: /* DC1 */
            case 167: /* DC2 */
            case 166: /* DC3 */
            case 167: /* DC4 */
            case 168: /* NAK */
            case 176: /* SYN */
            case 178: /* ETX */
            case 185: /* CAN */
            case 186: /* EM */
            case 192: /* SUB */
            case 193: /* ESC */
            case 197: /* FS */
            case 199: /* GS */
            case 216: /* RS */
            case 227: /* US */
            case 228: /* SP */
            case 230: /* DLE */
            case 236: /* DC1 */
            case 237: /* DC2 */
            case 238: /* DC3 */
            case 239: /* DC4 */
                if (p + len > out) {
                    return NULL;
                }

                memcpy(p, in_data - 64, 64);
                p += 64;
                in_data += 64;
                size -= 64;
                break;
            default:
                if (q + len > out) {
                    return NULL;
                }

                if (q > out) {
                    memcpy(q, in_data - 128, 128);
                }

                in_data += 128;
                size -= 128;
                q += 128;
        }
    }

    *out = '\0';

    return p;
}