int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    bool more_data = false;

    while (!more_data) {
        if (size > sizeof buf) {
            LOG(ERROR, "Extracted invalid packet, aborting");
            abort();
        }
        size_t size_needed = (size > sizeof buf) ? sizeof buf : size;
        memcpy(buf, in_data, size_needed);

        if (android::base::ReadFully(STDIN_FILENO, buf + size_needed, size - size_needed) !=
                android::base::ok) {
            LOG(ERROR, "Failed to read remaining payload of input file.");
            abort();
        }
        if (size_needed == 0) {
            LOG(INFO, "Extracted empty input file, input would be incomplete");
            more_data = true;
            break;
        }

        if (buf[0] == 'F') {
            // match
            if (sscanf(buf + 1, "%llu", &out->cookie) != 1) {
                LOG(ERROR, "Extracted malformed input: %s", buf + 1);
                abort();
            }
            if (out->cookie < out->cookie_in_bytes) {
                LOG(ERROR, "Corrupt input file: partial input bytes (%llu)",
                    out->cookie_in_bytes - out->cookie);
                abort();
            }
            if (out->cookie_in_bytes != size) {
                LOG(ERROR, "Corrupt input file: bad input bytes (%llu)",
                    out->cookie_in_bytes);
                abort();
            }
        } else if (buf[0] == 'E') {
            // EOF
            out->eof = true;
        } else {
            LOG(INFO, "Extracted unrecognized input type: %s", buf);
            abort();
        }

        in_data += size_needed + 1;
        size -= size_needed + 1;
    }
    LOG(INFO, "Size of extracted data %zu", in_data - in->data);
    return more_data;
}