int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];

    if(size <= 0) return -1;

    // We should ensure that in_data does not overrun buffer boundary.
    if(out == NULL || in_data == NULL || *out == '\0') return -2;

    *out = '\0';
    if(*in_data == 0x0a) {
        out_push_byte(*out);
        return 0;
    }

    size = 16;
    while(*in_data && size) {
        int64_t v = 0;
        size_t i = 0;

        // Read variable-length integer
        for(i = 0; i < 8; i++) {
            int8_t c = *in_data;

            if(c < 0) return -3;

            v <<= 7;
            v |= (c & 0x7f);

            in_data++;
            i++;
        }

        // Read remainder of byte sequence
        if(i < 7) {
            int8_t c = *in_data;

            if(c < 0) return -3;

            v <<= 7;
            v |= (c & 0x7f);

            in_data++;
            i++;
        }

        // Copy into the buffer
        memcpy(buf, &v, i * sizeof(v));

        // If i>=7 then we've got some partial or invalid
        if(i >= 7) return -4;

        // Pad with 0s
        buf[i] = '\0';

        // Output string until i is a multiple of 8 bytes
        out_push_byte(out_data(buf, i));

        size -= i;
    }

    // If there's no remainder we can't parse out
    return -5;
}