int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];

    if (size < 2) {
        fprintf(stderr, "not enough data for a MP4 file\n");
        return -1;
    }

    // some files use multiple headers at the beginning
    if (memcmp(in_data, "\xff\x2f\x00\x01", 4) != 0) {
        return -1;
    }

    // header is OK. lets do a line by line check and return if the line is not correct
    for (int i = 1; i < size; ++i) {
        if (in_data[i] == '\n') {
            break;
        }
        if (in_data[i] != ' ') {
            // something else unexpected is not we expecting a line break
            return -1;
        }
    }

    if (i == size) {
        // didn't get a line break in the first field
        return -1;
    }

    int r = snprintf(out, (size_t)size, "%s", in_data + 1);
    if (r < 0 || r > (int)size) {
        // definitely too big
        return -1;
    }

    // make sure the strlen is in bounds, fix to handle partial writes
    if (strlen(out) < (size_t)r) {
        return -1;
    }

    return 0;
}