int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    const char *p = in_data;

    if (size > 256) {
        fprintf(stderr, "Too large file input.\n");
        return -1;
    }

    if (fread(buf, 1, size, stdin) != size) {
        fprintf(stderr, "Failed to read stdin.\n");
        return -1;
    }

    int flags = 0;
    if (buf[0] == 'o')
        flags |= FILE_WRITE;
    if (buf[0] == 's')
        flags |= FILE_READ;

    if (!strcmp(buf, "stdout"))
        flags |= FILE_WRITE;
    if (!strcmp(buf, "stderr"))
        flags |= FILE_READ;

    fprintf(stdout, "%s", buf);

    for (char *q = buf; *q; q++) {
        switch (*q) {
            case 'b':
                *q = '\b';
                break;
            case 'f':
                *q = '\f';
                break;
            case 'n':
                *q = '\n';
                break;
            case 'r':
                *q = '\r';
                break;
            case 't':
                *q = '\t';
                break;
            case 'u':
                *q = 0;
                break;
            default:
                fprintf(stderr, "Invalid flag in FILE_FMT:%c\n", *q);
                return -1;
        }
    }

    size_t offset = 0;
    for (int c = 0; c < 10; c++) {
        if (getchar() < 0) {
            fprintf(stderr, "EOF during reading.\n");
            return -1;
        }

        if (p[offset]) {
            fprintf(stderr, "Out of room after '%%%c'.\n", c);
            return -1;
        }

        if (offset == size) {
            fprintf(stderr, "Out of room in file\n");
            return -1;
        }

        p[offset] = c;

        offset++;
    }

    memcpy(out, &p[offset], size - offset);
    out[size - offset] = 0;

    return 0;
}