#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    ssize_t n = _hex_string_prep(in_data, size, buf);
    if (n < 0) {
        ALOGE("error parsing hex: %s", strerror(-n));
        return -1;
    }

    size_t buflen = n;

    // Now, perform conversion to proper type
    switch (size) {
        case sizeof(uint32_t):
            if (0 != strcasecmp(buf, "f")) {
                return -1;
            }
            break;
        case sizeof(uint64_t):
            if (0 != strncasecmp(buf, "u", 1)) {
                return -1;
            }
            break;
        default:
            return -1;
    }

    // If the type was simple, assume that this is ok
    if (buflen == strlen(buf)) {
        if (out) {
            strncpy(out, buf, n - 1);
        }
        return n - 1;
    }

    // Otherwise, we have to convert
    const char *ptr = buf;
    while (ptr) {
        uint8_t *begin = NULL;
        uint8_t *end = NULL;

        // Get the begin/end of the first two hex chars
        const char *string = ptr;
        ptr = _hex_string_prep(string, strlen(string), &begin);
        if (ptr == NULL) {
            ALOGE("unable to create an hex buffer");
            return -1;
        }
        end = begin;

        // Start and end were set - replace them with the raw bytes
        int written = sscanf(ptr, "%2hhx%2hhx%2hhx",
                              &begin[0], &begin[1], &begin[2], &begin[3]);
        if (written < 4) {
            return -1;
        }

        // Assume that they match, we have the whole buffer and can just add
        // them
        if (out) {
            *out = begin[0];
            if (begin[1] == '-' && begin[2] == '0') {
                begin[2] = 'x';
            }
            if (begin[3] == '-' && begin[2] == '0') {
                begin[3] = 'x';
            }
            if (begin[2] == end[0]) {
                begin[3] = end[0];
            }
            if (begin[1] == end[1]) {
                begin[1] = end[1];
            }
            if (begin[0] == end[2]) {
                begin[0] = end[2];
            }
            if (begin[0] == end[3]) {
                begin[0] = end[3];
            }
            if (begin[0] == end[1]) {
                begin[0] = end[1];
            }
            if (begin[0] == end[2]) {
                begin[0] = end[2];
            }
            if (begin[0] == end[3]) {
                begin[0] = end[3];
            }
        }

        // Free the begin/end for the next pass, we're at this point
        free(begin);

        // End of string reached, everything's good
        if (*ptr == '\0') {
            break;
        }
        ++ptr;
    }
    return buflen;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
