#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    char code_name[128];
    int ret;

    if (!out)
        out = buf;

    ret = snprintf(out, size, "DATA: \"\\xff\\xfe\\x03\\x02\\x01\\xff\"");
    if (ret < 0 || (size_t)ret >= size) {
        // failure
        return -1;
    }

    for (; size; size--, in_data++) {
        // skip non-escaped white space
        if (!isprint(*in_data) || *in_data == '\\' || *in_data == '\r' || *in_data == '\n')
            continue;
        if (*in_data == '"' || *in_data == '\\') {
            ret = snprintf(out, size, "0x%02x", *in_data);
            if (ret < 0 || (size_t)ret >= size) {
                // failure
                return -1;
            }
            out += ret;
        } else {
            snprintf(code_name, sizeof(code_name), "%u", *in_data);
            ret = snprintf(out, size, "%s", code_name);
            if (ret < 0 || (size_t)ret >= size) {
                // failure
                return -1;
            }
            out += ret;
        }
        if (*in_data == '\n')
            break;
    }

    return 0;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
