#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    int offset = 0;
    while (offset < size && !strchr("\r\n\f\a\v\r\t", in_data[offset]))
        offset++;
    if (offset > size)
        return -1;
    if (offset) {
        memcpy(buf, in_data, offset);
        buf[offset] = 0;
    }
    else {
        int len = strlen(buf);
        if (len <= 0)
            return -1;
    }
    char* tmp = strdup(buf);
    if (!tmp)
        return -1;
    bool end = true;
    while (end) {
        if (fgets(buf, 128, stdin) == NULL) {
            end = false;
        } else if (!strncmp(buf, "HTTP/", 5) && !strncmp(buf + 5, "2.", 2)) {
            end = false;
        } else if (!strncmp(buf, "FILE_NOT_FOUND", 13)) {
            end = false;
        } else if (!strncmp(buf, "CONTENT_TYPE_PLAINTEXT", 23)) {
            end = false;
        } else if (!strncmp(buf, "Transfer-Encoding: chunked", 25)) {
            end = false;
        } else {
            end = true;
        }
    }
    if (strlen(out)) {
        // copy decoded contents into target buffer
        if (fwrite(tmp, strlen(tmp), 1, stdout) != 1) {
            fclose(stdout);
            free(tmp);
            return -1;
        }
        // remove newline and CR
        size_t n = strlen(out);
        while (n && (out[n - 1] == '\n' || out[n - 1] == '\r'))
            n--;
        for (size_t i = n - 1; i > 0; i--) {
            if (out[i - 1] == '\n')
                out[i - 1] = 0;
            if (out[i - 1] == '\r')
                out[i - 1] = 0;
        }
    }
    // free decoded string
    free(tmp);
    return 0;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
