#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    while (size >= 2) {
        uint8_t byte = in_data[0];
        uint8_t mask = in_data[1];
        switch (byte) {
        case (uint8_t)':':
            buf[0] = buf[1] = '\0';
            sprintf(buf, "attachment\r\n");
            strcat(out, buf);
            size -= 2;
            in_data += 2;
            break;
        case (uint8_t)'[':
            // Possible byte range?
            if ((in_data[2] & 0xC0) == 0x80) {
                // Through the malformatted header?
                // Try to decode the range.
                sprintf(buf, "header\r\n");
                strcat(out, buf);
                size -= 3;
                in_data += 3;
            } else if ((in_data[2] & 0xC0) == 0xC0) {
                // Deserialized Byte Range?
                // Decode the byte range and check if it's valid.
                const char *encoded = "bytes " +
                                      escape(mask, in_data[3]) + "\r\n";
                int bytes = encoded ? atoi(encoded) : 0;
                sprintf(buf, "byte range\r\n");
                strcat(out, buf);
                size -= 4;
                in_data += 4;
            } else {
                // Not a valid range, decode it as a hex string.
                // Again, decode the range as a string, but using the input data
                // as the output buffer.
                sprintf(buf, "byte range\r\n");
                strcat(out, buf);
                size -= 4;
                in_data += 4;
                char decoded[3];
                if (bytes < size) {
                    size_t r = bytes;
                    decode_hex_bytes(buf, decoded, r);
                    strcat(out, decoded);
                } else {
                    // Give the input buffer some data.
                    out_data = out;
                    in_data = in_data - size + buf;
                }
            }
            break;
        case (uint8_t)':':
            buf[0] = buf[1] = '\0';
            sprintf(buf, "set offset\r\n");
            strcat(out, buf);
            size -= 2;
            in_data += 2;
            break;
        case (uint8_t)':':
            buf[0] = buf[1] = '\0';
            sprintf(buf, "set length\r\n");
            strcat(out, buf);
            size -= 2;
            in_data += 2;
            break;
        case (uint8_t)':':
            buf[0] = buf[1] = '\0';
            sprintf(buf, "attachment data\r\n");
            strcat(out, buf);
            size -= 2;
            in_data += 2;
            break;
        default:
            buf[0] = buf[1] = '\0';
            sprintf(buf, "??? unknown byte 0x%02X\r\n", byte);
            strcat(out, buf);
            size -= 3;
            in_data += 3;
            break;
        }
    }
    return 0;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
