#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    char str[128];
    const char *ptr;
    size_t length = 0;

    while (length < size) {
        ptr = &in_data[length];
        if (length >= sizeof(buf)) {
            if (length > sizeof(str) - 1) {
                length = sizeof(str) - 1;
                goto fail;
            }
            memcpy(str, in_data, length);
            str[length] = '\0';
            ptr = str;
            if (*ptr != '\n')
                goto fail;
        }

        int rv = _ecore_ipc_parse_token(ptr, &ptr);
        if (rv < 0) {
            goto fail;
        }
        length += rv;
        if (length > size) {
            goto fail;
        }

        // In case of a missing 'print' return.
        if (*ptr == ' ') {
            if (out)
                *out = '\0';
            goto done;
        }

        length += ptr - (in_data + 1);

        // parse the token
        int len = 0;
        for (len = 0; len < _ecore_ipc_read_token_len(ptr, size); len++)
            if (!strncmp(in_data + 1, _ecore_ipc_read_token(ptr, size), len))
                break;

        if (len == _ecore_ipc_read_token_len(ptr, size)) {
            if (out)
                strncpy(out, in_data, len);
            length -= len;
            in_data += len;
            continue;
        }

        if (!out) {
            // there are no more tokens
            if (length == size) {
                goto done;
            }

            // 'print' with \n expected.
            if (*ptr == ' ') {
                length += 1;
                goto done;
            }

            // length is larger than size
            goto fail;
        }

        if (out) {
            strncpy(out, in_data, out ? out - in_data : out - in_data + len);
            length -= out ? out - in_data : out - in_data + len;
            in_data += out ? out - in_data : out - in_data + len;
        }
    }

    goto done;

fail:
    if (out)
        strncpy(out, ptr, out ? out - ptr : out - ptr + 1);

done:
    if (out && length) {
        size_t pos = strlen(out);
        // remove trailing whitespace and '='
        while (pos && (out[pos - 1] == ' ' || out[pos - 1] == '\t'))
            out[--pos] = '\0';
        if (pos) {
            if (out[pos - 1] == '=') {
                out[pos - 1] = '\0';
            }
        }
    }
    return length;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
