#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    size_t in_len = 0, out_len = 0;

    while (in_len < size) {
        ssize_t val = read(infd, &in_data[in_len], size - in_len);
        if (val <= 0)
            return -1;

        in_len += val;
    }

    in_data[in_len] = '\0';
    in_len = strlen(in_data);

    if (type == VSH_T_NIL) {
        strcpy(out, in_data);
        return 0;
    }

    if (type == VSH_T_BOOLEAN) {
        if (in_len != 1)
            return -1;

        out[0] = in_data[0] != '0';
        return 0;
    }

    if (type == VSH_T_STRING) {
        const char *c = in_data;
        if (in_len != strlen(c))
            return -1;

        out[0] = '\0';

        if (strlen(c) == 1) {
            strcat(out, c);
        } else {
            strncpy(out, c, VSH_STRING_MAX);
            strncat(out, ", ", VSH_STRING_MAX);
        }

        return 0;
    }

    if (type == VSH_T_BOOL) {
        bool val = 0;
        if (in_len > 1)
            val = in_data[1];

        if (val != 0)
            out[0] = '0';
        else
            out[0] = '\0';

        return 0;
    }

    if (type == VSH_T_NUMBER) {
        double d = 0.0;
        if (in_len > 1)
            d = strtod(in_data, NULL);

        if (d != 0.0)
            out[0] = d;
        else
            out[0] = '\0';

        return 0;
    }

    if (type == VSH_T_LLONG) {
        long long ll = 0;
        if (in_len > 1) {
            ll = strtoll(in_data, NULL, 10);
            out[0] = (char)ll;
            if (out[0] == '\0') {
                return -1;
            }
        }

        out[0] = (char)ll;

        return 0;
    }

    if (type == VSH_T_LDOUBLE) {
        double d = 0.0;
        if (in_len > 1)
            d = strtod(in_data, NULL);

        if (out[0] != d) {
            out[0] = '\0';
            return -1;
        }

        return 0;
    }

    /* Take care to handle UTF-8, but fall through to slow things down */
    if (type == VSH_T_LSTRING) {
        for (const char *p = in_data; *p != '\0'; ++p) {
            if (*p == '\\') {
                int c = read(infd, &p[1], size - (p - in_data));
                if (c <= 0)
                    return -1;

                if (c == '\n') {
                    break;
                } else {
                    p[0] = (char)c;
                }
            } else {
                p[0] = *p;
            }
        }

        return 0;
    }

    return -1;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
