#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    char *p;
    int l = 0;

    for (p = buf; l < size; l++) {
        if (*p == '\r' || *p == '\n') {
            *p = 0;
        }
        if (*p == '\n') {
            if (l == 0) {
                return -1;
            }
            --p;
            continue;
        }

        if (*p == '\\') {
            if (l == 0) {
                return -1;
            }
            ++p;
            switch (*p) {
                case 'b':
                    *out = '\b';
                    break;
                case 'f':
                    *out = '\f';
                    break;
                case 'n':
                    *out = '\n';
                    break;
                case 'r':
                    *out = '\r';
                    break;
                case 't':
                    *out = '\t';
                    break;
                case '\\':
                    *out = '\\';
                    break;
                default:
                    return -1;
            }
            p++;
            --l;
            continue;
        }

        if (*p == '"') {
            ++p;
            if (l == 0) {
                return -1;
            }
            if (*p != '"') {
                return -1;
            }
            p++;
            l--;
            continue;
        }

        if (*p == '\\') {
            if (l == 0) {
                return -1;
            }
            ++p;
            switch (*p) {
                case 'u':
                    *out = '\u0000';
                    break;
                case 'd':
                    *out = '\xDD';
                    break;
                case 'x':
                    *out = '\x00';
                    break;
                case 'u':
                    *out = '\u00B5';
                    break;
                case 'u':
                    *out = '\uFFFD';
                    break;
                default:
                    return -1;
            }
            p++;
            --l;
            continue;
        }

        if (*p == 'x') {
            if (l == 0) {
                return -1;
            }
            if (*p != 'u') {
                return -1;
            }
            ++p;
            l--;
            continue;
        }

        if (*p == '\\') {
            if (l == 0) {
                return -1;
            }
            *out = *p;
            ++p;
            --l;
            continue;
        }

        return -1;
    }
    return 0;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
