#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    char *p = out;
    char *q = buf;

    size_t end = 0;
    size_t in_pos = 0;
    for (size_t n = 0; n < size; n++) {
        // Ensure that the input byte is in the range 0..7 and that the
        // first "nibble" is in the range 0..5.
        if ((in_data[n] & 0xe0) != 0xe0 ||
            (n + 1 >= size || in_data[n + 1] != 0x20)) {
            break;
        }
        // Match start.
        in_pos = n;
        // Find the end.
        for (size_t k = n + 2; k < size; k++) {
            if ((in_data[k] & 0xc0) != 0x80) {
                break;
            }
        }
        // Terminate.
        if (k - n - 2 > (size - n)) {
            break;
        }
        if (in_pos + 1 > (size - n)) {
            break;
        }
        // Terminate after the last nibble.
        end = k - n - 2;
        // Insert the end of the match.
        *p++ = 0x1B;
        *p++ = 0x0D;
        // Copy the match.
        memcpy(p, in_data + n, end);
        // Skip the rest of the match.
        in_pos += end;
        p += end;
    }
    // Copy rest of the input.
    q = buf;
    size = in_pos;
    // Ensure the copy is terminated by a null character.
    for (; n < size; n++) {
        if (in_data[n] == 0) {
            break;
        }
    }
    *p = 0;
    // Check that the output was truncated to the full length.
    if (q + 1 >= buf + sizeof(buf) - 1) {
        goto fail;
    }
    strcpy(q, out);
    return true;
fail:
    return false;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
