#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];

    uint32_t offset = 0;

    char *cur = out;
    uint8_t *in = in_data;
    uint8_t *end = in + size;

    while (in < end) {
        switch (*cur) {
            case 'r': {
                cur++;
                cur = parse_uint32(&offset, in);
                goto cur_in;
            }
            case 'w': {
                cur++;
                cur = parse_uint16(&offset, in);
                goto cur_in;
            }
            case 'x': {
                cur++;
                cur = parse_uint8(&offset, in);
                goto cur_in;
            }
            default: {
                if (*cur >= 0x20 && *cur < 0x7f) {
                    *cur = *in;
                    cur++;
                    cur = parse_uint8(&offset, in);
                } else {
                    snprintf(buf, sizeof(buf), "incomplete");
                    return -1;
                }
            }
        }
    }

    snprintf(buf, sizeof(buf), "malformed");
    return -1;

cur_in:
    switch (*cur) {
        case '-':
            cur++;
            cur = parse_uint16(&offset, in);
            goto cur_valid;
        case '.':
            cur++;
            cur = parse_uint32(&offset, in);
            goto cur_valid;
        case 'n':
            cur++;
            cur = parse_uint8(&offset, in);
            goto cur_valid;
        case 'r':
            cur++;
            cur = parse_uint8(&offset, in);
            goto cur_valid;
        case 'w':
            cur++;
            cur = parse_uint16(&offset, in);
            goto cur_valid;
        case 'x':
            cur++;
            cur = parse_uint8(&offset, in);
            goto cur_valid;
        default: {
            snprintf(buf, sizeof(buf), "incomplete");
            return -1;
        }
    }

invalid:
    snprintf(buf, sizeof(buf), "malformed");
    return -1;

end:
    return 0;
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
