#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <glib.h>

int parse(const uint8_t *in_data, size_t size, char *out) {
    char buf[128];
    unsigned int in_len;

    if (out == NULL || size < 1)
        return -1;
    buf[0] = 0;
    in_len = strlen(in_data);
    if (in_len >= 128)
        return -1;
    if (in_len == 0)
        return 0;
    strcpy(out, in_data);
    buf[in_len] = '\0';

    /* split string at '=' character.
     * can't use strtok(3) because that is supposed to process the
     * attribute name, and doesn't work correctly when there are
     * spaces in the key (ie: empty values).
     * also makes sure we do not assume that we are in a comment.
     */
    char *cur = out;
    for (;;) {
        char *start, *end;
        if (buf[0] == 0) {
            //buffer is empty.
            return -1;
        }
        start = strstr(buf, "=");
        if (start == NULL) {
            //key or value not in table, process it as normal string.
            buf[0] = 0;
            buf[1] = 0;
            cur = out;
        } else {
            //the value equals the key, ignore it.
            end = strstr(start, "=\"");
            if (end == NULL) {
                cur = start;
            } else {
                start = end;
                buf[0] = 0;
            }
        }
        if (in_len == 1 && *buf == '=')
            return 0;
        else if (in_len == 2 && buf[0] == ':' && buf[1] == '=' && buf[2] == '=')
            return 0;
        else if (in_len == 3 && buf[0] == ':' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=')
            return 0;
        else if (in_len == 4 && buf[0] == '<' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=')
            return 0;
        else if (in_len == 5 && buf[0] == '=' && buf[1] == '<' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=' && buf[5] == '=')
            return 0;
        else if (in_len == 6 && buf[0] == '<' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=' && buf[5] == '=' && buf[6] == '=')
            return 0;
        else if (in_len == 7 && buf[0] == '>' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=' && buf[5] == '=' && buf[6] == '=')
            return 0;
        else if (in_len == 8 && buf[0] == '>' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=' && buf[5] == '=' && buf[6] == '=' && buf[7] == '=')
            return 0;
        else if (in_len == 9 && buf[0] == '<' && buf[1] == '=' && buf[2] == '=' && buf[3] == '=' && buf[4] == '=' && buf[5] == '=' && buf[6] == '=' && buf[7] == '=' && buf[8] == '=')
            return 0;
        else {
            cur = start + in_len + 1;
        }
    }
}
int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
    char *buf = malloc(Size);
    int n = parse(Data, Size, buf);
    free(buf);
    return n;
}
